/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "serverclient.h"

#define SERVER_CLIENT_COUNT_MAX 8
#define SERVER_PORT_DEFAULT 24555

typedef enum {
  SERVER_STATE_STOPPED,
  SERVER_STATE_STARTING,
  SERVER_STATE_RUNNING,
  SERVER_STATE_STOPPING
} serverstate_t;

typedef enum {
  SERVER_TYPE_OFFLINE,
  SERVER_TYPE_ONLINE
} servertype_t;

typedef struct {
  serverstate_t state;
  servertype_t type;
  serverclient_t clients[SERVER_CLIENT_COUNT_MAX];
  uint8_t clientCount;
  pthread_t thread;

  union {
    struct {
      int32_t socket;
      struct sockaddr_in address;
    } online;
  };
} server_t;

typedef struct {
  servertype_t type;

  union {
    struct {
      int32_t port;
    } online;
  };
} serverinit_t;

extern server_t SERVER;

void serverInit(const serverinit_t init);
void * serverThread(void* arg);
void serverDispose();